<sect> Tree Widget <label id="sec_Tree_Widgets">
<!-- ***************************************************************** -->
<p>
The purpose of tree widgets is to display hierarchically-organized
data. The Gtk_Tree widget itself is a vertical container for widgets of
type Gtk_TreeItem. Gtk_Tree itself is not terribly different from
Gtk_List - both are derived directly from Gtk_Container, and the
Gtk_Container methods work in the same way on Gtk_Tree widgets as on
Gtk_List widgets. The difference is that Gtk_Tree widgets can be nested
within other Gtk_Tree widgets. We'll see how to do this shortly.

The Gtk_Tree widget has its own window, and defaults to a white
background, as does Gtk_List. Also, most of the Gtk_Tree methods work in
the same way as the corresponding Gtk_List ones. However, Gtk_Tree is
not derived from Gtk_List, so you cannot use them interchangeably.

<sect1> Creating a Tree
<p>
A Gtk_Tree is created in the usual way, using:

<tscreen><verb>
Gtk_Tree();
</verb></tscreen>

Like the Gtk_List widget, a Gtk_Tree will simply keep growing as more
items are added to it, as well as when subtrees are expanded.  For
this reason, they are almost always packed into a
Gtk_ScrolledWindow. You might want to use Gtk_Widget::set_usize() on the
scrolled window to ensure that it is big enough to see the tree's
items, as the default size for Gtk_ScrolledWindow is quite small.

Now that you have a tree, you'll probably want to add some items to
it.  <ref id="sec_Tree_Item_Widget" name="The Tree Item Widget"> below
explains the gory details of Gtk_TreeItem. For now, it'll suffice to
create one, using:

<tscreen><verb>
Gtk_TreeItem( const string &amp;label );
</verb></tscreen>

You can then add it to the tree using one of the following (see
<ref id="sec_Gtk_Tree_Functions" name="Methods">
below for more options):

<tscreen><verb>
void Gtk_Tree::append( const Gtk_Widget &amp;tree_item );

void Gtk_Tree::prepend( const Gtk_Widget &amp;tree_item );
</verb></tscreen>

Note that you must add items to a Gtk_Tree one at a time - there is no
equivalent to Gtk_List::*_items().

<!-- ----------------------------------------------------------------- -->
<sect1> Adding a Subtree
<p>
A subtree is created like any other Gtk_Tree widget. A subtree is added
to another tree beneath a tree item, using:

<tscreen><verb>
void Gtk_TreeItem::set_subtree(const Gtk_Tree &amp;subtree );
</verb></tscreen>

You do not need to call Gtk_Widget::show() on a subtree before or after
adding it to a Gtk_TreeItem. However, you <em>must</em> have added the
Gtk_TreeItem in question to a parent tree before calling
Gtk_TreeItem::set_subtree(). This is because, technically, the parent
of the subtree is <em>not</em> the Gtk_TreeItem which "owns" it, but
rather the Gtk_Tree which holds that Gtk_TreeItem.

When you add a subtree to a Gtk_TreeItem, a plus or minus sign appears
beside it, which the user can click on to "expand" or "collapse" it,
meaning, to show or hide its subtree. Gtk_TreeItems are collapsed by
default. Note that when you collapse a Gtk_TreeItem, any selected
items in its subtree remain selected, which may not be what the user
expects.

<!-- ----------------------------------------------------------------- -->
<sect1> Handling the Selection List
<p>
As with Gtk_List, the Gtk_Tree type has a <tt>selection</tt> field, and
it is possible to control the behaviour of the tree (somewhat) by
setting the selection type using:

<tscreen><verb>
void Gtk_Tree::set_selection_mode(GtkSelectionMode  mode );
</verb></tscreen>

The semantics associated with the various selection modes are
described in the section on the Gtk_List widget. As with the Gtk_List
widget, the "select_child", "unselect_child" (not really - see <ref
id="sec_Gtk_Tree_Signals" name="Signals"> below for an explanation),
and "selection_changed" signals are emitted when list items are
selected or unselected. However, in order to take advantage of these
signals, you need to know <em>which</em> Gtk_Tree widget they will be
emitted by, and where to find the list of selected items.

This is a source of potential confusion. The best way to explain this
is that though all Gtk_Tree widgets are created equal, some are more
equal than others. All Gtk_Tree widgets have their own X window, and
can therefore receive events such as mouse clicks (if their
Gtk_TreeItems or their children don't catch them first!). However, to
make GTK_SELECTION_SINGLE and GTK_SELECTION_BROWSE selection types
behave in a sane manner, the list of selected items is specific to the
topmost Gtk_Tree widget in a hierarchy, known as the "root tree".

Thus, accessing the <tt>selection</tt>field directly in an arbitrary
Gtk_Tree widget is not a good idea unless you <em>know</em> it's the
root tree.  Instead, use the GTK_TREE_SELECTION (Tree) macro, which
gives the root tree's selection list as a GList pointer. Of course,
this list can include items that are not in the subtree in question if 
the selection type is GTK_SELECTION_MULTIPLE.

Finally, the "select_child" (and "unselect_child", in theory) signals
are emitted by all trees, but the "selection_changed" signal is only
emitted by the root tree. Consequently, if you want to handle the
"select_child" signal for a tree and all its subtrees, you will have
to call gtk_signal_connect() for every subtree.

<sect1> Tree Widget Internals
<p>
The Gtk_Tree's struct definition looks like this:

<tscreen><verb>
struct _Gtk_Tree
{
  GtkContainer container;

  GList *children;
  
  Gtk_Tree* root_tree; /* owner of selection list */
  GtkWidget* tree_owner;
  GList *selection;
  guint level;
  guint indent_value;
  guint current_indent;
  guint selection_mode : 2;
  guint view_mode : 1;
  guint view_line : 1;
};
</verb></tscreen>

The perils associated with accessing the <tt>selection</tt> field
directly have already been mentioned. The other important fields of
the struct can also be accessed with handy macros or class functions.
GTK_TREE_IS_ROOT_TREE (Tree) returns a boolean value which indicates
whether a tree is the root tree in a Gtk_Tree hierarchy, while
GTK_TREE_ROOT_TREE (Tree) returns the root tree, an object of type
Gtk_Tree (so, remember to cast it using GTK_WIDGET (Tree) if you want
to use one of the gtk_widget_*() functions on it).

Instead of directly accessing the children field of a Gtk_Tree widget,
it's probably best to cast it using GTK_CONTAINER (Tree), and pass it
to the gtk_container_children() function. This creates a duplicate of
the original list, so it's advisable to free it up using g_list_free()
after you're done with it, or to iterate on it destructively, like
this:

<tscreen><verb>
    children = gtk_container_children (GTK_CONTAINER (tree));
    while (children) {
      do_something_nice (GTK_TREE_ITEM (children->data));
      children = g_list_remove_link (children, children);
}
</verb></tscreen>

The <tt>tree_owner</tt> field is defined only in subtrees, where it
points to the Gtk_TreeItem widget which holds the tree in question.
The <tt>level</tt> field indicates how deeply nested a particular tree
is; root trees have level 0, and each successive level of subtrees has
a level one greater than the parent level. This field is set only
after a Gtk_Tree widget is actually mapped (i.e. drawn on the screen).

<sect2> Signals<label id="sec_Gtk_Tree_Signals">
<p>
<tscreen><verb>
void selection_changed( Gtk_Tree *tree );
</verb></tscreen>

This signal will be emitted whenever the <tt>selection</tt> field of a
Gtk_Tree has changed. This happens when a child of the Gtk_Tree is
selected or deselected.

<tscreen><verb>
void select_child( Gtk_Tree   *tree,
                   GtkWidget *child );
</verb></tscreen>

This signal is emitted when a child of the Gtk_Tree is about to get
selected. This happens on calls to Gtk_Tree::select_item(),
Gtk_Tree::select_child(), on <em>all</em> button presses and calls to
Gtk_Item::toggle().  It may sometimes be indirectly triggered on other
occasions where children get added to or removed from the Gtk_Tree.

<tscreen><verb>
void unselect_child (Gtk_Tree   *tree,
                     GtkWidget *child);
</verb></tscreen>

This signal is emitted when a child of the Gtk_Tree is about to get
deselected. As of Gtk-- 1.0.4, this seems to only occur on calls to
gtk_tree_unselect_item() or gtk_tree_unselect_child(), and perhaps on
other occasions, but <em>not</em> when a button press deselects a
child, nor on emission of the "toggle" signal by gtk_item_toggle().

<sect2> Methods<label id="sec_Gtk_Tree_Functions">
<p>

<tscreen><verb>
Gtk_Tree();
</verb></tscreen>

Gtk_Tree ctor

<tscreen><verb>
void Gtk_Tree::append(const Gtk_Widget &amp;tree_item );
</verb></tscreen>

Append a tree item to a Gtk_Tree.

<tscreen><verb>
void Gtk_Tree::prepend(const Gtk_Widget &amp;tree_item );
</verb></tscreen>

Prepend a tree item to a Gtk_Tree.

<tscreen><verb>
void Gtk_Tree::insert(const Gtk_Widget &amp;tree_item,
                      gint       position );
</verb></tscreen>

Insert a tree item into a Gtk_Tree at the position in the list
specified by <tt>position.</tt>

<tscreen><verb>
!!!!!!!!!!
void Gtk_Tree::remove_items(GList   *items );
</verb></tscreen>

Remove a list of items (in the form of a GList *) from a Gtk_Tree.
Note that removing an item from a tree dereferences (and thus usually)
destroys it <em>and</em> its subtree, if it has one, <em>and</em> all
subtrees in that subtree.  If you want to remove only one item, you
can use gtk_container_remove().

<tscreen><verb>
void Gtk_Tree::clear_items(gint     start,
                           gint     end );
</verb></tscreen>

Remove the items from position <tt>start</tt> to position <tt>end</tt>
from a Gtk_Tree.  The same warning about dereferencing applies here, as
Gtk_Tree::clear_items() simply constructs a list and passes it to
Gtk_Tree::remove_items().

<tscreen><verb>
void Gtk_Tree::select_item(gint     item );
</verb></tscreen>

Emits the "select_item" signal for the child at position
<tt>item</tt>, thus selecting the child (unless you unselect it in a
signal handler).

<tscreen><verb>
void Gtk_Tree::unselect_item(gint     item );
</verb></tscreen>

Emits the "unselect_item" signal for the child at position
<tt>item</tt>, thus unselecting the child.

<tscreen><verb>
void Gtk_Tree::select_child(const Gtk_Widget &amp;tree_item );
</verb></tscreen>

Emits the "select_item" signal for the child <tt>tree_item</tt>, thus
selecting it.

<tscreen><verb>
void Gtk_Tree::unselect_child(const Gtk_Widget &amp;tree_item );
</verb></tscreen>

Emits the "unselect_item" signal for the child <tt>tree_item</tt>,
thus unselecting it.

<tscreen><verb>
gint Gtk_Tree::child_position(const Gtk_Widget &amp;child );
</verb></tscreen>

Returns the position in the tree of <tt>child</tt>, unless
<tt>child</tt> is not in the tree, in which case it returns -1.

<tscreen><verb>
void Gtk_Tree::set_selection_mode(GtkSelectionMode  mode );
</verb></tscreen>

Sets the selection mode, which can be one of GTK_SELECTION_SINGLE (the
default), GTK_SELECTION_BROWSE, GTK_SELECTION_MULTIPLE, or
GTK_SELECTION_EXTENDED. This is only defined for root trees, which
makes sense, since the root tree "owns" the selection. Setting it for
subtrees has no effect at all; the value is simply ignored.

<tscreen><verb>
void Gtk_Tree::set_view_mode(GtkTreeViewMode  mode ); 
</verb></tscreen>

Sets the "view mode", which can be either GTK_TREE_VIEW_LINE (the
default) or GTK_TREE_VIEW_ITEM.  The view mode propagates from a tree
to its subtrees, and can't be set exclusively to a subtree (this is
not exactly true - see the example code comments).

The term "view mode" is rather ambiguous - basically, it controls the
way the highlight is drawn when one of a tree's children is selected.
If it's GTK_TREE_VIEW_LINE, the entire Gtk_TreeItem widget is
highlighted, while for GTK_TREE_VIEW_ITEM, only the child widget
(i.e. usually the label) is highlighted.

<tscreen><verb>
void Gtk_Tree::set_view_lines(guint    flag );
</verb></tscreen>

Controls whether connecting lines between tree items are drawn.
<tt>flag</tt> is either TRUE, in which case they are, or FALSE, in
which case they aren't.

!!!!!!!!!! wrap these
<tscreen><verb>
gint GTK_IS_ROOT_TREE (gpointer obj)
</verb></tscreen>

Determine if a generic pointer refers to a `Gtk_Tree' object
<em>and</em> is a root tree. Though this will accept any pointer, the
results of passing it a pointer that does not refer to a Gtk_Tree are
undefined and possibly harmful.

<tscreen><verb>
Gtk_Tree *GTK_TREE_ROOT_TREE (gpointer obj)
</verb></tscreen>

Return the root tree of a pointer to a `Gtk_Tree' object. The above
warning applies.

<tscreen><verb>
GList *GTK_TREE_SELECTION( gpointer obj)
</verb></tscreen>

Return the selection list of the root tree of a `Gtk_Tree' object. The
above warning applies here, too.

<sect1> Tree Item Widget<label id="sec_Tree_Item_Widget">
<p>
The Gtk_TreeItem widget, like Gtk_ListItem, is derived from GtkItem,
which in turn is derived from GtkBin.  Therefore, the item itself is a
generic container holding exactly one child widget, which can be of
any type.  The Gtk_TreeItem widget has a number of extra fields, but
the only one we need be concerned with is the <tt>subtree</tt> field.

The definition for the Gtk_TreeItem struct looks like this:

<tscreen><verb>
struct _Gtk_TreeItem
{
  GtkItem item;

  GtkWidget *subtree;
  GtkWidget *pixmaps_box;
  GtkWidget *plus_pix_widget, *minus_pix_widget;

  GList *pixmaps;		/* pixmap node for this items color depth */

  guint expanded : 1;
};
</verb></tscreen>

The <tt>pixmaps_box</tt> field is a GtkEventBox which catches clicks
on the plus/minus symbol which controls expansion and collapsing.  The
<tt>pixmaps</tt> field points to an internal data structure.  Since
you can always obtain the subtree of a Gtk_TreeItem in a (relatively)
type-safe manner with the GTK_TREE_ITEM_SUBTREE (Item) macro, it's
probably advisable never to touch the insides of a Gtk_TreeItem unless
you <em>really</em> know what you're doing.

Since it is directly derived from a GtkItem it can be treated as such
by using the GTK_ITEM (TreeItem) macro. A Gtk_TreeItem usually holds a
label, so the convenience function gtk_list_item_new_with_label() is
provided. The same effect can be achieved using code like the
following, which is actually copied verbatim from
gtk_tree_item_new_with_label():

<tscreen><verb>
tree_item = gtk_tree_item_new ();
label_widget = gtk_label_new (label);
gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);

gtk_container_add (GTK_CONTAINER (tree_item), label_widget);
gtk_widget_show (label_widget);
</verb></tscreen>

As one is not forced to add a GtkLabel to a Gtk_TreeItem, you could
also add a GtkHBox or a GtkArrow, or even a GtkNotebook (though your
app will likely be quite unpopular in this case) to the Gtk_TreeItem.

If you remove all the items from a subtree, it will be destroyed and
unparented, unless you reference it beforehand, and the Gtk_TreeItem
which owns it will be collapsed.  So, if you want it to stick around,
do something like the following:

<tscreen><verb>
gtk_widget_ref (tree);
owner = GTK_TREE(tree)->tree_owner;
gtk_container_remove (GTK_CONTAINER(tree), item);
if (tree->parent == NULL){
  gtk_tree_item_expand (GTK_TREE_ITEM(owner));
  gtk_tree_item_set_subtree (GTK_TREE_ITEM(owner), tree);
}
else
  gtk_widget_unref (tree);
</verb></tscreen>

Finally, drag-n-drop <em>does</em> work with Gtk_TreeItems.  You just
have to make sure that the Gtk_TreeItem you want to make into a drag
item or a drop site has not only been added to a Gtk_Tree, but that
each successive parent widget has a parent itself, all the way back to
a toplevel or dialog window, when you call gtk_widget_dnd_drag_set()
or gtk_widget_dnd_drop_set().  Otherwise, strange things will happen.

<sect2> Signals
<p>
Gtk_TreeItem inherits the "select", "deselect", and "toggle" signals
from GtkItem.  In addition, it adds two signals of its own, "expand"
and "collapse".

<tscreen><verb>
void select( GtkItem *tree_item );
</verb></tscreen>

This signal is emitted when an item is about to be selected, either
after it has been clicked on by the user, or when the program calls
gtk_tree_item_select(), gtk_item_select(), or gtk_tree_select_child().

<tscreen><verb>
void deselect( GtkItem *tree_item );
</verb></tscreen>

This signal is emitted when an item is about to be unselected, either
after it has been clicked on by the user, or when the program calls
gtk_tree_item_deselect() or gtk_item_deselect(). In the case of
Gtk_TreeItems, it is also emitted by gtk_tree_unselect_child(), and
sometimes gtk_tree_select_child().

<tscreen><verb>
void toggle( GtkItem *tree_item );
</verb></tscreen>

This signal is emitted when the program calls gtk_item_toggle().  The
effect it has when emitted on a Gtk_TreeItem is to call
gtk_tree_select_child() (and never gtk_tree_unselect_child()) on the
item's parent tree, if the item has a parent tree.  If it doesn't,
then the highlight is reversed on the item.

<tscreen><verb>
void expand( Gtk_TreeItem *tree_item );
</verb></tscreen>

This signal is emitted when the tree item's subtree is about to be
expanded, that is, when the user clicks on the plus sign next to the
item, or when the program calls gtk_tree_item_expand().

<tscreen><verb>
void collapse( Gtk_TreeItem *tree_item );
</verb></tscreen>

This signal is emitted when the tree item's subtree is about to be
collapsed, that is, when the user clicks on the minus sign next to the
item, or when the program calls gtk_tree_item_collapse().

<sect2> Functions and Macros
<p>
<tscreen><verb>
guint gtk_tree_item_get_type( void );
</verb></tscreen>

Returns the `Gtk_TreeItem' type identifier.

<tscreen><verb>
GtkWidget* gtk_tree_item_new( void );
</verb></tscreen>

Create a new Gtk_TreeItem object. The new widget is returned as a
pointer to a GtkWidget object. NULL is returned on failure.

<tscreen><verb>
GtkWidget* gtk_tree_item_new_with_label (gchar       *label);
</verb></tscreen>

Create a new Gtk_TreeItem object, having a single GtkLabel as the sole
child. The new widget is returned as a pointer to a GtkWidget
object. NULL is returned on failure.

<tscreen><verb>
void gtk_tree_item_select( Gtk_TreeItem *tree_item );
</verb></tscreen>

This function is basically a wrapper around a call to gtk_item_select
(GTK_ITEM (tree_item)) which will emit the select signal.

<tscreen><verb>
void gtk_tree_item_deselect( Gtk_TreeItem *tree_item );
</verb></tscreen>

This function is basically a wrapper around a call to
gtk_item_deselect (GTK_ITEM (tree_item)) which will emit the deselect
signal.

<tscreen><verb>
void gtk_tree_item_set_subtree( Gtk_TreeItem *tree_item,
                                GtkWidget   *subtree );
</verb></tscreen>

This function adds subtree to tree_item, showing it if tree_item is
expanded, or hiding it if tree_item is collapsed. Again, remember that
the tree_item must have already been added to a tree for this to work.

<tscreen><verb>
void gtk_tree_item_remove_subtree( Gtk_TreeItem *tree_item );
</verb></tscreen>

This removes all of tree_item's subtree's children (thus unreferencing
and destroying it, any of its children's subtrees, and so on...), then
removes the subtree itself, and hides the plus/minus sign.

<tscreen><verb>
void gtk_tree_item_expand( Gtk_TreeItem *tree_item );
</verb></tscreen>

This emits the "expand" signal on tree_item, which expands it.

<tscreen><verb>
void gtk_tree_item_collapse( Gtk_TreeItem *tree_item );
</verb></tscreen>

This emits the "collapse" signal on tree_item, which collapses it.

<tscreen><verb>
Gtk_TreeItem *GTK_TREE_ITEM (gpointer obj)
</verb></tscreen>

Cast a generic pointer to `Gtk_TreeItem*'.

<tscreen><verb>
Gtk_TreeItemClass *GTK_TREE_ITEM_CLASS (gpointer obj)
</verb></tscreen>

Cast a generic pointer to `Gtk_TreeItemClass'.

<tscreen><verb>
gint GTK_IS_TREE_ITEM (gpointer obj)
</verb></tscreen>

Determine if a generic pointer refers to a `Gtk_TreeItem' object.
 
<tscreen><verb>
GtkWidget GTK_TREE_ITEM_SUBTREE (gpointer obj)
</verb></tscreen>

Returns a tree item's subtree (obj should point to a `Gtk_TreeItem'
object).

<sect1> Tree Example
<p>
This is somewhat like the tree example in testgtk.c, but a lot less
complete (although much better commented).  It puts up a window with a
tree, and connects all the signals for the relevant objects, so you
can see when they are emitted.

<tscreen><verb>
example_include(tree/tree.cc)
</verb></tscreen>

<!-- ***************************************************************** -->
